@@ -14,10 +14,12 @@ A discord.py extension including useful tools for bot development and debugging.
 # pylint: disable=wildcard-import
 from jishaku.cog import *  # noqa: F401
 from jishaku.features.baseclass import Feature  # noqa: F401
+from jishaku.flags import Flags  # noqa: F401
 from jishaku.meta import *  # noqa: F401
 
 __all__ = (
     'Jishaku',
     'Feature',
+    'Flags',
     'setup'
 )
