@@ -11,7 +11,9 @@ Meta information about jishaku.
 
 """
 
-from collections import namedtuple
+import typing
+
+import pkg_resources
 
 __all__ = (
     '__author__',
@@ -23,9 +25,17 @@ __all__ = (
     'version_info'
 )
 
-# pylint: disable=invalid-name
-VersionInfo = namedtuple('VersionInfo', 'major minor micro releaselevel serial')
-version_info = VersionInfo(major=2, minor=0, micro=0, releaselevel='final', serial=0)
+
+class VersionInfo(typing.NamedTuple):
+    """Version info named tuple for Jishaku"""
+    major: int
+    minor: int
+    micro: int
+    releaselevel: str
+    serial: int
+
+
+version_info = VersionInfo(major=2, minor=5, micro=1, releaselevel='final', serial=0)
 
 __author__ = 'Gorialis'
 __copyright__ = 'Copyright 2021 Devon (Gorialis) R'
@@ -33,3 +43,6 @@ __docformat__ = 'restructuredtext en'
 __license__ = 'MIT'
 __title__ = 'jishaku'
 __version__ = '.'.join(map(str, (version_info.major, version_info.minor, version_info.micro)))
+
+# This ensures that when jishaku is reloaded, pkg_resources requeries it to provide correct version info
+pkg_resources.working_set.by_key.pop('jishaku', None)  # type: ignore
