@@ -14,14 +14,29 @@ Functions for managing and searching modules.
 import pathlib
 import typing
 
+import discord
 import pkg_resources
-from braceexpand import UnbalancedBracesError, braceexpand
+from braceexpand import braceexpand
 from discord.ext import commands
 
+from jishaku.types import BotT, ContextA
+
 __all__ = ('find_extensions_in', 'resolve_extensions', 'package_version', 'ExtensionConverter')
 
 
-def find_extensions_in(path: typing.Union[str, pathlib.Path]) -> list:
+if typing.TYPE_CHECKING:
+    UnbalancedBracesError = ValueError
+else:
+    from braceexpand import UnbalancedBracesError
+
+
+if typing.TYPE_CHECKING or discord.version_info >= (2, 0, 0):
+    _ExtensionConverterBase = commands.Converter[typing.List[str]]
+else:
+    _ExtensionConverterBase = commands.Converter
+
+
+def find_extensions_in(path: typing.Union[str, pathlib.Path]) -> typing.List[str]:
     """
     Tries to find things that look like bot extensions in a directory.
     """
@@ -32,7 +47,7 @@ def find_extensions_in(path: typing.Union[str, pathlib.Path]) -> list:
     if not path.is_dir():
         return []
 
-    extension_names = []
+    extension_names: typing.List[str] = []
 
     # Find extensions directly in this folder
     for subpath in path.glob('*.py'):
@@ -53,12 +68,12 @@ def find_extensions_in(path: typing.Union[str, pathlib.Path]) -> list:
     return extension_names
 
 
-def resolve_extensions(bot: commands.Bot, name: str) -> list:
+def resolve_extensions(bot: BotT, name: str) -> typing.List[str]:
     """
     Tries to resolve extension queries into a list of extension names.
     """
 
-    exts = []
+    exts: typing.List[str] = []
     for ext in braceexpand(name):
         if ext.endswith('.*'):
             module_parts = ext[:-2].split('.')
@@ -83,12 +98,20 @@ def package_version(package_name: str) -> typing.Optional[str]:
         return None
 
 
-class ExtensionConverter(commands.Converter):  # pylint: disable=too-few-public-methods
+class ExtensionConverter(_ExtensionConverterBase):  # pylint: disable=too-few-public-methods
     """
     A converter interface for resolve_extensions to match extensions from users.
     """
 
-    async def convert(self, ctx: commands.Context, argument) -> list:
+    async def convert(
+        self,
+        ctx: ContextA,
+        argument: str
+    ) -> typing.List[str]:
+        """
+        Converts a name, glob, or brace expand of extensions into the list of extension names.
+        """
+
         try:
             return resolve_extensions(ctx.bot, argument)
         except UnbalancedBracesError as exc:
