@@ -13,7 +13,7 @@ Repl-related operations and tools for Jishaku.
 
 # pylint: disable=wildcard-import
 from jishaku.repl.compilation import *  # noqa: F401
-from jishaku.repl.disassembly import disassemble  # noqa: F401
-from jishaku.repl.inspections import all_inspections  # noqa: F401
-from jishaku.repl.repl_builtins import get_var_dict_from_ctx  # noqa: F401
+from jishaku.repl.disassembly import create_tree, disassemble  # type: ignore  # noqa: F401
+from jishaku.repl.inspections import all_inspections  # type: ignore  # noqa: F401
+from jishaku.repl.repl_builtins import get_var_dict_from_ctx  # type: ignore  # noqa: F401
 from jishaku.repl.scope import *  # noqa: F401
