@@ -15,17 +15,18 @@ import ast
 import asyncio
 import inspect
 import linecache
+import typing
 
-import import_expression
+import import_expression  # type: ignore
 
 from jishaku.functools import AsyncSender
 from jishaku.repl.scope import Scope
 from jishaku.repl.walkers import KeywordTransformer
 
-CORO_CODE = """
+CORO_CODE = f"""
 async def _repl_coroutine({{0}}):
     import asyncio
-    from importlib import import_module as {0}
+    from importlib import import_module as {import_expression.constants.IMPORTER}
 
     import aiohttp
     import discord
@@ -40,18 +41,22 @@ async def _repl_coroutine({{0}}):
         pass
     finally:
         _async_executor.scope.globals.update(locals())
-""".format(import_expression.constants.IMPORTER)
+"""
 
 
-def wrap_code(code: str, args: str = '') -> ast.Module:
+def wrap_code(code: str, args: str = '', auto_return: bool = True) -> ast.Module:
     """
     Compiles Python code into an async function or generator,
     and automatically adds return if the function body is a single evaluation.
     Also adds inline import expression support.
     """
 
-    user_code = import_expression.parse(code, mode='exec')
-    mod = import_expression.parse(CORO_CODE.format(args), mode='exec')
+    user_code: ast.Module = import_expression.parse(code, mode='exec')  # type: ignore
+    mod: ast.Module = import_expression.parse(CORO_CODE.format(args), mode='exec')  # type: ignore
+
+    for node in ast.walk(mod):
+        node.lineno = -100_000
+        node.end_lineno = -100_000
 
     definition = mod.body[-1]  # async def ...:
     assert isinstance(definition, ast.AsyncFunctionDef)
@@ -65,6 +70,10 @@ def wrap_code(code: str, args: str = '') -> ast.Module:
 
     KeywordTransformer().generic_visit(try_block)
 
+    # if auto return is disabled, we're done here
+    if not auto_return:
+        return mod
+
     last_expr = try_block.body[-1]
 
     # if the last part isn't an expression, ignore it
@@ -106,9 +115,17 @@ class AsyncCodeExecutor:  # pylint: disable=too-few-public-methods
         print(total)
     """
 
-    __slots__ = ('args', 'arg_names', 'code', 'loop', 'scope', 'source')
+    __slots__ = ('args', 'arg_names', 'code', 'loop', 'scope', 'source', '_function')
 
-    def __init__(self, code: str, scope: Scope = None, arg_dict: dict = None, loop: asyncio.BaseEventLoop = None):
+    def __init__(
+        self,
+        code: str,
+        scope: typing.Optional[Scope] = None,
+        arg_dict: typing.Optional[typing.Dict[str, typing.Any]] = None,
+        convertables: typing.Optional[typing.Dict[str, str]] = None,
+        loop: typing.Optional[asyncio.BaseEventLoop] = None,
+        auto_return: bool = True,
+    ):
         self.args = [self]
         self.arg_names = ['_async_executor']
 
@@ -118,17 +135,69 @@ class AsyncCodeExecutor:  # pylint: disable=too-few-public-methods
                 self.args.append(value)
 
         self.source = code
-        self.code = wrap_code(code, args=', '.join(self.arg_names))
+
+        try:
+            self.code = wrap_code(code, args=', '.join(self.arg_names), auto_return=auto_return)
+        except (SyntaxError, IndentationError) as first_error:
+            if not convertables:
+                raise
+
+            try:
+                for key, value in convertables.items():
+                    code = code.replace(key, value)
+                self.code = wrap_code(code, args=', '.join(self.arg_names))
+            except (SyntaxError, IndentationError) as second_error:
+                raise second_error from first_error
+
         self.scope = scope or Scope()
         self.loop = loop or asyncio.get_event_loop()
+        self._function = None
+
+    @property
+    def function(self) -> typing.Callable[..., typing.Union[
+        typing.Awaitable[typing.Any],
+        typing.AsyncGenerator[typing.Any, typing.Any]
+    ]]:
+        """
+        The function object produced from compiling the code.
+        If the code has not been compiled yet, it will be done upon first access.
+        """
+
+        if self._function is not None:
+            return self._function
 
-    def __aiter__(self):
         exec(compile(self.code, '<repl>', 'exec'), self.scope.globals, self.scope.locals)  # pylint: disable=exec-used
-        func_def = self.scope.locals.get('_repl_coroutine') or self.scope.globals['_repl_coroutine']
+        self._function = self.scope.locals.get('_repl_coroutine') or self.scope.globals['_repl_coroutine']
 
-        return self.traverse(func_def)
+        return self._function
 
-    async def traverse(self, func):
+    def create_linecache(self) -> typing.List[str]:
+        """
+        Populates the line cache with the current source.
+        Can be performed before printing a traceback to show correct source lines.
+        """
+
+        lines = [line + '\n' for line in self.source.splitlines()]
+
+        linecache.cache['<repl>'] = (
+            len(self.source),  # Source length
+            None,  # Time modified (None bypasses expunge)
+            lines,  # Line list
+            '<repl>'  # 'True' filename
+        )
+
+        return lines
+
+    def __aiter__(self) -> typing.AsyncGenerator[typing.Any, typing.Any]:
+        return self.traverse(self.function)
+
+    async def traverse(
+        self,
+        func: typing.Callable[..., typing.Union[
+            typing.Awaitable[typing.Any],
+            typing.AsyncGenerator[typing.Any, typing.Any]
+        ]]
+    ) -> typing.AsyncGenerator[typing.Any, typing.Any]:
         """
         Traverses an async function or generator, yielding each result.
 
@@ -137,17 +206,14 @@ class AsyncCodeExecutor:  # pylint: disable=too-few-public-methods
 
         try:
             if inspect.isasyncgenfunction(func):
-                async for send, result in AsyncSender(func(*self.args)):
+                func_g: typing.Callable[..., typing.AsyncGenerator[typing.Any, typing.Any]] = func  # type: ignore
+                async for send, result in AsyncSender(func_g(*self.args)):  # type: ignore
                     send((yield result))
             else:
-                yield await func(*self.args)
+                func_a: typing.Callable[..., typing.Awaitable[typing.Any]] = func  # type: ignore
+                yield await func_a(*self.args)
         except Exception:  # pylint: disable=broad-except
             # Falsely populate the linecache to make the REPL line appear in tracebacks
-            linecache.cache['<repl>'] = (
-                len(self.source),  # Source length
-                None,  # Time modified (None bypasses expunge)
-                [line + '\n' for line in self.source.splitlines()],  # Line list
-                '<repl>'  # 'True' filename
-            )
+            self.create_linecache()
 
             raise
